﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
//using System.Threading.Tasks;
using System.Windows.Forms;
using Phidget22;
using Phidget22.Events;
using Phidget22.ExampleUtils;
using System.IO;

namespace Encoder_Example {
	public partial class Form1 : Form {
		CommandLineOpen open;
        formCleaner cleaner;
        Phidget22.Encoder encoder = null;
		private ErrorEventBox errorBox;
		int CPR = -1;

        private DataGraphBox positionGraphBox, velocityGraphBox;

		public Form1() {
			open = new CommandLineOpen(this);
            cleaner = new formCleaner(this);
            InitializeComponent();
		}

		public Form1(String[] commandLine) {
			open = new CommandLineOpen(this);
            cleaner = new formCleaner(this);
            open.commandLine = commandLine;
			InitializeComponent();
		}

		private void Form1_Load(object sender, EventArgs e) {
			phidgetInfoBox1.Clear();

			
			errorBox = new ErrorEventBox(this);
			encoder = new Phidget22.Encoder();

            positionGraphBox = new DataGraphBox(this, "Position");
			velocityGraphBox = new DataGraphBox(this, "Velocity");

			encoder.Attach += enc_attach;
			encoder.Detach += enc_detach;
			encoder.Error += enc_error;
			encoder.PositionChange += enc_change;


            commandLineData phidgetParameters = open.parseCmdLine(); //get command line parameters

            try
            { //set all the values grabbed from command line.  these values have defaults that are set in ExampleUtils.cs, you can check there to see them.
                encoder.Channel = phidgetParameters.Channel; //selects the channel on the device to open
                encoder.DeviceSerialNumber = phidgetParameters.SerialNumber; //selects the device or hub to open
                encoder.HubPort = phidgetParameters.HubPort; //selects the port on the hub to open
                encoder.IsHubPortDevice = phidgetParameters.isHubPortDevice; //is the device a port on a VINT hub?

                if (phidgetParameters.isRemote) //are we trying to open a remote device?
                {
                    encoder.IsRemote = true;
                    Net.EnableServerDiscovery(ServerType.Device); //turn on network scan
                    if (phidgetParameters.Password != null && phidgetParameters.ServerName != null)
                        Net.SetServerPassword(phidgetParameters.ServerName, phidgetParameters.Password); //set the password if there is one
                }
                else
                    encoder.IsLocal = true;

                encoder.Open(); //open the device specified by the above parameters
            }
            catch (PhidgetException ex) { errorBox.addMessage("Error opening device: " + ex.Message); }
        }

		private void Form1_FormClosing(object sender, FormClosingEventArgs e) {
			encoder.Close();
		}
		void enc_attach(object sender, Phidget22.Events.AttachEventArgs e) {
			phidgetInfoBox1.FillPhidgetInfo((Phidget)sender);

			Phidget22.Encoder attachedDevice = (Phidget22.Encoder)sender;

			try {
				dataIntervalTrk.Unit = "ms";
				dataIntervalTrk.trkText = dataIntervalTrk.minTxt = attachedDevice.MinDataInterval.ToString();
				dataIntervalTrk.Minimum = attachedDevice.MinDataInterval;
				dataIntervalTrk.maxTxt = attachedDevice.MaxDataInterval.ToString();
				dataIntervalTrk.Maximum = attachedDevice.MaxDataInterval;
				dataIntervalTrk.Value = attachedDevice.DataInterval;

				//differing max/min means the interval is a settable configuration
				if (attachedDevice.MaxDataInterval == attachedDevice.MinDataInterval)
					dataIntervalTrk.Enabled = false;

				changeTriggerTrk.Unit = " ticks";
				changeTriggerTrk.trkText = changeTriggerTrk.minTxt = attachedDevice.MinPositionChangeTrigger.ToString();
				changeTriggerTrk.Minimum = attachedDevice.MinPositionChangeTrigger;
				changeTriggerTrk.Maximum = attachedDevice.MaxPositionChangeTrigger;
				changeTriggerTrk.maxTxt = attachedDevice.MaxPositionChangeTrigger.ToString();
				changeTriggerTrk.Value = attachedDevice.PositionChangeTrigger;
				changeTriggerTrk.trkText = attachedDevice.PositionChangeTrigger.ToString("F0") + changeTriggerTrk.Unit;

			} catch (PhidgetException ex) {
				errorBox.addMessage("Error setting data interval or change trigger: " + ex.Message);
			}
            if (encoder.DeviceClass != DeviceClass.VINT)
            {
				iomodePanel.Hide();
			}
			dataInterval = attachedDevice.MinDataInterval;
			timeTxt.Clear();
			positionTxt.Clear();
			indexTxt.Clear();
			cumulativePosition.Clear();
			timeTxt.Text = "N/A";
			positionTxt.Text = "N/A";
			indexTxt.Text = "N/A";
			cumulativePosition.Text = "N/A";

			switch (((Phidget)sender).DeviceID) {
			case DeviceID.PN_1047:
			case DeviceID.PN_HIN1101:
                enabledChk.Visible = true;
				enabledChk.Checked = true;
				attachedDevice.Enabled = true;
				iomodePanel.Hide();
				break;
			case DeviceID.PN_ENC1000:
			case DeviceID.PN_DCC1000:
				foreach (EncoderIOMode eio in Enum.GetValues(typeof(EncoderIOMode)))
					iomodeCombo.Items.Add(eio);
				iomodeCombo.SelectedIndex = 0;
				break;
            case DeviceID.PN_DCC1002:
                iomodePanel.Hide();
                break;
            }

			settingsBox.Visible = true;
			outputBox.Visible = true;
		}
		void enc_detach(object sender, Phidget22.Events.DetachEventArgs e) {
			phidgetInfoBox1.Clear();
            cleaner.clean();

            settingsBox.Visible = false;
			outputBox.Visible = false;

			enabledChk.Visible = false;
			cprTxt.Text = "";
			CPR = -1;
		}
		void enc_error(object sender, Phidget22.Events.ErrorEventArgs e) {
			errorBox.addMessage(e.Description);
		}

		void enc_change(object sender, Phidget22.Events.EncoderPositionChangeEventArgs e) {
			try {
				positionTxt.Text = e.PositionChange.ToString() + " ticks";
				timeTxt.Text = (e.TimeChange).ToString("F3") + " ms";

				if (e.IndexTriggered) {
					indexTxt.Text = encoder.IndexPosition.ToString() + " ticks";
				}

				cumulativePosition.Text = encoder.Position.ToString() + " ticks";

				if (CPR != -1) {
					double timeChangeMinutes = e.TimeChange / 60000.0; // ms -> minutes
					double velocity = (((double)e.PositionChange / CPR) / timeChangeMinutes);
					rpmTxt.Text = velocity.ToString("F0");

					velocityGraphBox.addData(velocity, encoder.DataInterval);
				}

                positionGraphBox.addData(encoder.Position, encoder.DataInterval);
			} catch (PhidgetException ex) {
				errorBox.addMessage("Error reading position: " + ex.Message);
			}
		}

		private void enabledChk_CheckedChanged(object sender, EventArgs e) {
			try {
				encoder.Enabled = enabledChk.Checked;
			} catch (PhidgetException ex) {
				errorBox.addMessage("Error setting enabled: " + ex.Message);
			}
		}

		private void changeTriggertrk_Scroll(object sender, EventArgs e) {
			if (MouseButtons != MouseButtons.Left) {
				try {
					encoder.PositionChangeTrigger = (int)changeTriggerTrk.Value;
				} catch (Exception ex) {
					errorBox.addMessage("Error setting change trigger: " + ex.Message);
				}
			}
		}

		private void changeTriggertrk_MouseUp(object sender, MouseEventArgs e) {
			try {
				encoder.PositionChangeTrigger = (int)changeTriggerTrk.Value;
			} catch (Exception ex) {
				errorBox.addMessage("Error setting change trigger: " + ex.Message);
			}
		}

		private void cprTxt_KeyDown(object sender, KeyEventArgs e) {
			if (e.KeyCode == Keys.Enter) {
				try {
					if (cprTxt.Text != "" && cprTxt.Text != "0")
						CPR = Int32.Parse(cprTxt.Text);
					else {
						CPR = -1;
						rpmTxt.Text = "N/A";
						cprTxt.Text = "";
					}
				} catch (Exception) {
					errorBox.addMessage("Invalid CPR value.");
					cprTxt.Text = "";
				}
				e.SuppressKeyPress = true;
			}
		}

		private void CPRSetBtn_Click(object sender, EventArgs e) {
			try {
				CPR = Int32.Parse(cprTxt.Text);
			} catch (Exception) {
				errorBox.addMessage("Invalid CPR value.");
				cprTxt.Text = "";
			}
		}

		private void rstBtn_Click(object sender, EventArgs e) {
			try {
				encoder.Position = 0;
                positionGraphBox.addData(0, encoder.DataInterval);
			} catch (Exception ex) {
				errorBox.addMessage("Error setting position: " + ex.Message);
			}
		}

		int dataInterval = 0;
		private void dataIntTrk_Scroll(object sender, EventArgs e) {
			if (MouseButtons != MouseButtons.Left) {
				try {
					encoder.DataInterval = (int)dataIntervalTrk.Value;
				} catch (Exception ex) {
					errorBox.addMessage("Error setting change trigger: " + ex.Message);
				}
			}
		}

		private void dataIntTrk_MouseUp(object sender, MouseEventArgs e) {
			try {
				encoder.DataInterval = (int)dataIntervalTrk.Value;
			} catch (Exception ex) {
				errorBox.addMessage("Error setting data interval: " + ex.Message);
			}
		}

		private void iomodeCombo_SelectedIndexChanged(object sender, EventArgs e) {
			try {
				encoder.IOMode = (EncoderIOMode)iomodeCombo.Items[iomodeCombo.SelectedIndex];
			} catch (Exception ex) {
				errorBox.addMessage("Error setting IO Mode: " + ex.Message);
			}
		}

        private void positionGraphButton_Click(object sender, EventArgs e)
        {
            if (!positionGraphBox.Visible)
                positionGraphBox.Show();
        }

		private void dataIntervalTrk_SetClick(object sender, EventArgs e)
		{
			try
			{
				encoder.DataInterval = (int)dataIntervalTrk.Value;
			}
			catch (Exception ex) { errorBox.addMessage("Error setting change trigger: " + ex.Message); }
		}

		private void dataIntervalTrk_SetEnter(object sender, KeyEventArgs e)
		{
			if (e.KeyCode == Keys.Enter)
			{
				try
				{
					encoder.DataInterval = (int)dataIntervalTrk.Value;
				}
				catch (Exception ex) { errorBox.addMessage("Error setting change trigger: " + ex.Message); }
			}
		}

		private void changeTriggerTrk_SetClick(object sender, EventArgs e)
		{
			try
			{
				encoder.PositionChangeTrigger = (int)changeTriggerTrk.Value;
			}
			catch (Exception ex) { errorBox.addMessage("Error setting change trigger: " + ex.Message); }
		}

		private void changeTriggerTrk_SetEnter(object sender, KeyEventArgs e)
		{
			if (e.KeyCode == Keys.Enter)
			{
				try
				{
					encoder.PositionChangeTrigger = (int)changeTriggerTrk.Value;
				}
				catch (Exception ex) { errorBox.addMessage("Error setting change trigger: " + ex.Message); }
			}
		}

        private void velocityGraphButton_Click(object sender, EventArgs e)
        {
			if (!velocityGraphBox.Visible)
				velocityGraphBox.Show();
		}
    }
}
